/* Copyright Amit P. Singh, Douglas L. Brutlag, Stanford University. */

#include <stdio.h> 
#include <stdlib.h> 
#include "defines.h" 

#ifdef DISPLAYON
#include <GL/gl.h> 
#endif

char usageErrorMessage[] = {
"\n\nlock usage:\n \
\t-q  <pdbName1> \t\tquery pdb file name OR pdb code\n \
\t-qc <chain1>   \t\tquery chain identifier (if present)\n \
\t-t  <pdbName2> \t\ttarget pdb file name OR pdb code\n \
\t-tc <chain2>   \t\ttarget chain identifier (if present)\n \
\t-b             \t\trun in batch mode without graphical interface\n \
\t-n             \t\tdo not compute core alignment\n \
\t-o  <dirName>  \t\toutput directory name\n \
\t-d  <threshold>\t\tthreshold value\n \
\t-h             \t\tprint this help \n \
\n\
Note: \
\tThe query and target can be specified as either complete file names\n \
\tor as pdb accession codes.  If the pdb codes are used, you can also\n \
\tspecify a chain by concatenating the chain id to the pdb code as follows:\n \
\tlock -q 1seb-A -t 1seb-B \n\n"
};


 
 
char threelettercode[22][4] = {  
  {"ALA"}, {"ARG"}, {"ASN"}, {"ASP"}, {"ASX"}, {"CYS"}, {"GLN"}, {"GLU"}, {"GLX"}, {"GLY"}, {"HIS"}, 
  {"ILE"}, {"LEU"}, {"LYS"}, {"MET"}, {"PHE"}, {"PRO"}, {"SER"}, {"THR"}, {"TRP"}, {"TYR"}, {"VAL"}
};

char onelettercode[23] = {"ARNDBCQEZGHILKMFPSTWYV"};


char username[2][300];
char filename[2][300];
char pdbname[2][10] = {{"query"}, {"target"}};
char chain[2]; 
int newfile[2];
FILE* filePtrs[2]; 
char outputDir[300];

extern int globalStartDisplay;
extern int globalComputeCore;
extern float distThreshold; 
extern int mypid; 
extern int globalSearchDatabase;
extern FILE *globalDatabaseFile;
extern FILE *rasmolFilePtr;
extern char* lockdir;
extern char* dsspprogram;
 
extern OBJ* ObjList[2]; 
extern void freeObj(OBJ*);

#ifdef DISPLAYON
extern GLfloat colorTable[2][3];
extern void createObjList(OBJ*); 
#endif

OBJ* objectFromFile(int); 
OBJ* objectFromPDBFile(int); 
OBJ* objectFromDSSPFile(int); 
OBJ* readFile(int); 
OBJ* noSSinPDBFile(int); 

void checkopts(int, char**); 
void usageError(); 
void extStr(char*, char*, int, int); 
char convert3to1Code(char*);
void convert1to3Code(char, char*);
int checkIfList(FILE*);

void usageError() { 
 
  printf(usageErrorMessage); 
  exit(1); 
} 
 

char convert3to1Code(char *code) {

  char ch;
  int i;

  ch = '*';

  for (i = 0; i < 22; i++) {
    if (strcmp(code, threelettercode[i]) == 0) {
      ch = onelettercode[i];
      break;
    }
  }
  return ch;
}

void convert1to3Code(char ch, char *retcode) {

  int i;

  strcpy(retcode, "NUL");

  for (i = 0; i < 22; i++) {
    if ((onelettercode[i] == ch) || (((char)((int)(onelettercode[i]) - 32)) == ch)) {
      strcpy(retcode, threelettercode[i]);
      break;
    }
  }
    
}

 
void checkopts(int argc, char **argv) { 
 
  int i;  
 
  strcpy(username[0], "NONE");
  chain[0] = ' '; 
  strcpy(username[1], "NONE");
  chain[1] = ' '; 
  strcpy(outputDir, "NONE");
 
  for (i = 0; i < argc; i++) { 
 
    if (strcmp(argv[i], "-q") == 0) { 
 
      if (i < argc - 1) { 
	if (argv[i+1][0] != '-') {	   
	  strcpy(username[0], argv[i+1]); 
	  i++; 
	} 
	else { 
	  usageError(); 
	} 
      } 
      else { 
	usageError(); 
      } 
 
    } 
 
    else if (strcmp(argv[i], "-qc") == 0) { 
       
      if (i < argc - 1) { 
 
	if (argv[i+1][0] != '-') { 
	  chain[0] = argv[i+1][0]; 
	  i++; 
	} 
	else { 
	  usageError(); 
	} 
      } 
      else { 
	usageError(); 
      } 
 
    } 
 
    else if (strcmp(argv[i], "-t") == 0) { 
 
      if (i < argc - 1) { 
	if (argv[i+1][0] != '-') {	   
	  strcpy(username[1], argv[i+1]); 
	  i++; 
	} 
	else { 
	  usageError(); 
	} 
      } 
      else { 
	usageError(); 
      } 
 
    } 
 
    else if (strcmp(argv[i], "-tc") == 0) { 
       
      if (i < argc - 1) { 
 
	if (argv[i+1][0] != '-') { 
	  chain[1] = argv[i+1][0]; 
	  i++; 
	} 
	else { 
	  usageError(); 
	} 
      } 
      else { 
	usageError(); 
      } 
 
    } 
 
    else if (strcmp(argv[i], "-d") == 0) { 
       
      if (i < argc - 1) { 
 
	if (argv[i+1][0] != '-') { 
	  sscanf(argv[i+1], "%f", &distThreshold); 
	  i++; 
	} 
	else { 
	  usageError(); 
	} 
      } 
      else { 
	usageError(); 
      } 
 
    } 

    else if (strcmp(argv[i], "-o") == 0) { 
       
      if (i < argc - 1) { 
 
	if (argv[i+1][0] != '-') { 
	  strcpy(outputDir, argv[i+1]); 
	  i++; 
	} 
	else { 
	  usageError(); 
	} 
      } 
      else { 
	usageError(); 
      } 
 
    } 

    else if (strcmp(argv[i], "-b") == 0) { 

      globalStartDisplay = 0;
 
    } 

    else if (strcmp(argv[i], "-n") == 0) { 

      globalComputeCore = 0;
 
    } 
 
    else if (strcmp(argv[i], "-h") == 0) { 
      usageError(); 
    } 
  } 
  
} 
 
 
 
int checkIfList(FILE *fp) {

  char line[200];
  char word[20];
  int retvalue, done;

  rewind(fp);

  done = 0;
  retvalue = 0;
  while ((fgets(line, 200, fp)) && (done == 0)) {
    if ((line[0] != '#') && (sscanf(line, "%s", word) > 0)) {
      done = 1;
 
      if (strcmp(word, "LOCK_LIST") == 0) {	
	retvalue = 1;
      }
    }
  }
  
  rewind(fp);

  return retvalue;
}


OBJ* readFile(int fnum) { 
 
  OBJ* tempObj;
  int len;
  char syscommand[200];
  int i, imax, count;

  filePtrs[fnum] = fopen(username[fnum], "r");

  if (filePtrs[fnum] != NULL)  {
    if ((fnum == 1) && (globalSearchDatabase == 0)) {
      if (checkIfList(filePtrs[fnum])) {
	globalSearchDatabase = 1;
	globalStartDisplay = 0;
	globalDatabaseFile = filePtrs[fnum];
	return NULL;
      }
    }
    strcpy(filename[fnum], username[fnum]);
    if (fnum == 0) {
      strcpy(pdbname[0], "query");
    }
    else if (fnum == 1) {
      strcpy(pdbname[1], "target");
    }
  

    if ((fnum == 1) && (globalSearchDatabase == 1)) {
      imax = strlen(username[1]);
      count = 0;
      
      for(i = 0; i < imax; i++) {
	if (username[1][i] != '/') {	
	  pdbname[1][count] = username[1][i];
	  count++;
	}
	else {
	  count = 0;
	}
      }
      pdbname[1][count] = '\0';
    }

    newfile[fnum] = 1;
    return objectFromFile(fnum); 
  }    
      
  else {

    len = strlen(username[fnum]);
    
    if (len == 4) {
      strcpy(pdbname[fnum], username[fnum]);   
      if (chain[fnum] != ' ') {
	pdbname[fnum][4] = '-';
	pdbname[fnum][5] = chain[fnum];
	pdbname[fnum][6] = '\0';
      }
    }

    else if ((len == 6) && (username[fnum][4] == '-')) {
      
      strcpy(pdbname[fnum], username[fnum]);

      if (chain[fnum] == ' ') {
	chain[fnum] = pdbname[fnum][5];
      }
      else if (chain[fnum] != pdbname[fnum][5]) {
	pdbname[fnum][5] = chain[fnum];
      }

    }

    else {      
      printf("Sorry.  Could not find file %s.\n\n", username[fnum]);
      return NULL;
    }

    newfile[fnum] = 0;
    filename[fnum][0] = '\0';
    sprintf(filename[fnum], "/tmp/%d.file%d", mypid, fnum);

    strcpy(syscommand, lockdir);
    strcat(syscommand, "findfile.pl ");
    strcat(syscommand, pdbname[fnum]);
    strcat(syscommand, " ");
    strcat(syscommand, filename[fnum]);

    system(syscommand);

    filePtrs[fnum] = fopen(filename[fnum], "r");

    if (filePtrs[fnum] != NULL) {
      tempObj = objectFromFile(fnum);

      if (filePtrs[fnum] != NULL) {
	strcpy(syscommand, "rm ");
	strcat(syscommand, filename[fnum]);
	system(syscommand);
      }
      return tempObj;
  
    }
    else {      
      printf("Sorry.  Could not find file %s.\n", username[fnum]);    
      return NULL;
    }      
 
  } 
 
}
  

OBJ* objectFromDSSPFile(int fnum) { 

  FILE *infile;
  OBJ *myobject;
  int i, j, XX, YY, ZZ;
  char line[200], temp[100], record[10];
  int match, arraylen, atomcount;
  float t0, t1, t2, tempfloat;
  int done = 0;

  infile = filePtrs[fnum];
  rewind(infile); 
 

  myobject = (OBJ*) malloc(sizeof(OBJ)); 
  myobject->compound[0] = '\0'; 
  myobject->objSeq = (char *) malloc((sizeof(char))*(MAXLENGTH));
  myobject->sstype = (char *) malloc((sizeof(char))*(MAXLENGTH)); 
  myobject->ssArray = (int*) malloc((sizeof(int))*(MAXLENGTH)); 

  fgets(line, 200, infile);

  while (((line[2] != '#') || (line[13] != 'A') || (line[14] != 'A') || (line[26] != 'B') || (line[27] != 'P') || (line[28] != '1')) && (done == 0)){

    extStr(record, line, 1, 6); 
 
    if (strcmp(record, "HEADER") == 0) { 
 
      extStr(myobject->header, line, 11, 50); 
 
    } 
 
    else if (strcmp(record, "COMPND") == 0) { 
 
      extStr(myobject->compound, line, 11, 70); 
        
    } 

    if (!(fgets(line, 200, infile))) 
      done = 1;
      
  }

  XX = -1;

  if (line[118] == 'X') {
    XX = 116;
    YY = 123;
    ZZ = 130;
  }
  else if (line[110] == 'X') {
    XX = 108;
    YY = 115;
    ZZ = 122;
  }
  else {
    for (i = 0; i < 190; i++) {
      if ((line[i] == 'X') && (line[i+1] == '-') && (line[i+2] == 'C') && (line[i+3] == 'A')) {
	XX = i-2;
	YY = XX + 7;
	ZZ = YY + 7;
	break;
      }
    }
  }

  if ((XX == -1) || (done == 1)) {
    printf("\nSorry.  The dssp file %s does not appear to be in the right format.\n\n", username[fnum]);
    fclose(infile);
    return NULL;
  }

  else {

    atomcount = 0;

    while (fgets(line, 200, infile)) {
      
      if (((chain[fnum] == ' ') || (line[11] == chain[fnum])) && (line[13] != '!')) {
	
	extStr(temp, line, XX, XX+6);
	sscanf(temp, "%f", &(myobject->points[atomcount][0]));

	extStr(temp, line, YY, YY+6);
	sscanf(temp, "%f", &(myobject->points[atomcount][1]));

	extStr(temp, line, ZZ, ZZ+6);
	sscanf(temp, "%f", &(myobject->points[atomcount][2]));	

	extStr(temp, line, 6, 10);
	sscanf(temp, "%d", &(myobject->aaNums[atomcount]));	

	myobject->objSeq[atomcount] = line[13];

	if (line[16] == 'H') {
	  myobject->sstype[atomcount] = 'H';
	}
	else if  (line[16] == 'G') {
	  myobject->sstype[atomcount] = 'G';
	}
	else if (line[16] == 'I') { 
	  myobject->sstype[atomcount] = 'I';
	}
	else if (line[16] == 'E') {
	  myobject->sstype[atomcount] = 'S';
	}
	else {
	  myobject->sstype[atomcount] = 'U';
	}
	  
	atomcount++;
      }
    }	

    if (atomcount == 0) {

      printf("\nSorry. No CA atoms with chain identifier \"%c\" found in file %s\n\n", chain[fnum], username[fnum]);
      fclose(infile);
      return NULL;
      
    }

    else {

      myobject->numPts = atomcount;      
      
      myobject->objSeq[atomcount] = '\0';  
      myobject->sstype[atomcount] = '\0';

      arraylen = 0;
      match = 1;

      for (i = 1; i < atomcount; i++) {

	if ((myobject->sstype[i] == myobject->sstype[i-1]) && (myobject->sstype[i] != 'U')) {
	  match++;
	}

	else {
	  if ((myobject->sstype[i-1] == 'H') && (match >= 4)) {
	    myobject->ssArray[arraylen*2] = i - match;
	    myobject->ssArray[arraylen*2 + 1] = i;
	    arraylen++;
	  }
	  else if ((myobject->sstype[i-1] == 'G') && (match >= 4)) {
	    myobject->ssArray[arraylen*2] = i - match;
	    myobject->ssArray[arraylen*2 + 1] = i;
	    arraylen++;
	    for (j = i-match; j < i; j++) {
	      myobject->sstype[j] = 'H';
	    }
	  }
	  else if ((myobject->sstype[i-1] == 'I') && (match >= 4)) {
	    myobject->ssArray[arraylen*2] = i - match;
	    myobject->ssArray[arraylen*2 + 1] = i;
	    arraylen++;
	    for (j = i-match; j < i; j++) {
	      myobject->sstype[j] = 'H';
	    }
	  }
	  else if ((myobject->sstype[i-1] == 'S') && (match >= 3)) {
	    myobject->ssArray[arraylen*2] = i - match;
	    myobject->ssArray[arraylen*2 + 1] = i;
	    arraylen++;
	  }
	  else {
	    for (j = i-match; j < i; j++) {
	      myobject->sstype[j] = 'U';
	    }
	  }
	  match = 1;
	}
      }

      myobject->ssArrayLength = arraylen;
      
      t0 = 0.0;
      t1 = 0.0;
      t2 = 0.0;
      
      for (i = 0; i < myobject->numPts; i++) {
	t0 += myobject->points[i][0];
	t1 += myobject->points[i][1];
	t2 += myobject->points[i][2];
      
	myobject->pairNums[i] = -1;
      
      }
      
      tempfloat = 1.0/((GLfloat)(myobject->numPts));
      
      tempfloat *= -1.0;
      
      t0 *= tempfloat;
      t1 *= tempfloat;
      t2 *= tempfloat;
      
      myobject->objMat[0] = 1.0;
      myobject->objMat[1] = 0.0;
      myobject->objMat[2] = 0.0;
      myobject->objMat[3] = 0.0;

      myobject->objMat[4] = 0.0;
      myobject->objMat[5] = 1.0;
      myobject->objMat[6] = 0.0;
      myobject->objMat[7] = 0.0;

      myobject->objMat[8] = 0.0;
      myobject->objMat[9] = 0.0;
      myobject->objMat[10] = 1.0;
      myobject->objMat[11] = 0.0;

      myobject->objMat[12] = t0;
      myobject->objMat[13] = t1;
      myobject->objMat[14] = t2;
      myobject->objMat[15] = 1.0;

      myobject->center[0] = 0.0;
      myobject->center[1] = 0.0;
      myobject->center[2] = 0.0;

#ifdef DISPLAYON
      if (globalStartDisplay) {

	myobject->modelNumber = glGenLists(1);
	myobject->color[0] = colorTable[fnum][0];
	myobject->color[1] = colorTable[fnum][1];
	myobject->color[2] = colorTable[fnum][2];	
	
	createObjList(myobject);
      }
#endif

      fclose(infile); 
      return myobject;     
    }
  }    
} 
 
void extStr(char* out, char* in, int st, int end) { 
 
  int outcount = 0; 
  int i;

  for (i = st - 1; i < end; i++) { 
 
    out[outcount] = in[i]; 
    outcount++; 
  } 
 
  out[outcount] = '\0'; 
 
} 
 
 
OBJ* objectFromPDBFile(int fnum) { 
 
  FILE* infile; 
  OBJ *myobject; 
  char record[10]; 
  char line[200]; 
  char temp[100]; 
  char atom[10];
  int i, j, k, readingDone = 0, arraylen;
  int atomCount = 0; 
  int resChains[MAXLENGTH]; 
  int helices[MAXSS][2]; 
  int sheets[MAXSS][2]; 
  int helixChains[MAXSS]; 
  int sheetChains[MAXSS]; 
  int numhelices = 0; 
  int numsheets = 0; 
  float t0 = 0.0, t1 = 0.0, t2 = 0.0, tempfloat;
  int count, gotss = 0;
  int gotCompound = 0;


  infile = filePtrs[fnum]; 
 
  myobject = (OBJ*) malloc(sizeof(OBJ)); 
 
  rewind(infile); 
 
  myobject->compound[0] = '\0'; 
  myobject->objSeq = (char *) malloc((sizeof(char))*(MAXLENGTH));


 
  while ((fgets(line, 200, infile)) && (readingDone == 0)) { 

    extStr(record, line, 1, 6); 
 
    if (strcmp(record, "HEADER") == 0) { 
 
      extStr(myobject->header, line, 11, 50); 
 
    } 
 
    else if ((strcmp(record, "COMPND") == 0) && (gotCompound == 0)) { 
 
      extStr(myobject->compound, line, 11, 70);  
      gotCompound = 1;
      
    } 
 
    else if (strcmp(record, "HELIX ") == 0) { 

 
      if ((chain[fnum] == ' ') || ((line[19] == chain[fnum]) && (line[31] == chain[fnum]))) { 
 
	gotss = 1;

	extStr(temp, line, 22, 25); 
	sscanf(temp, "%d", &(helices[numhelices][0])); 
 
	extStr(temp, line, 34, 37); 
	sscanf(temp, "%d", &(helices[numhelices][1])); 
	 
	helixChains[numhelices] = line[19];
	
	numhelices++; 
      }      
 
    } 
 
    else if (strcmp(record, "SHEET ") == 0) { 

      if ((chain[fnum] == ' ') || ((line[21] == chain[fnum]) && (line[32] == chain[fnum]))) { 
 
	gotss = 1;

	extStr(temp, line, 23, 26); 
	sscanf(temp, "%d", &(sheets[numsheets][0])); 
 
	extStr(temp, line, 34, 37); 
	sscanf(temp, "%d", &(sheets[numsheets][1])); 

	sheetChains[numsheets] = line[21];

	numsheets++; 
      }      
 
    } 
 
    else if (strcmp(record, "MODEL ") == 0) { 
      printf("NOTE:\tFound multiple models in the PDB file %s. \n\tWill use first model.\n",  username[fnum]); 
    } 
 
    else if (strcmp(record, "ATOM  ") == 0) { 


      if (gotss == 0) {

	free(myobject->objSeq);   
	free(myobject);
	fclose(infile); 
 
	return noSSinPDBFile(fnum); 
      }

      else { 
	 
	extStr(temp, line, 13, 16); 
	sscanf(temp, "%s", atom); 
 
	if ((atom[0] == 'C') && (atom[1] == 'A') && (strlen(atom) == 2)) { 
	  if ((chain[fnum] == ' ') || (line[21] == chain[fnum])) { 
	    extStr(temp, line, 18, 20); 
	    myobject->objSeq[atomCount] = convert3to1Code(temp); 
	     
	    extStr(temp, line, 31, 38); 
	    sscanf(temp, "%f", &(myobject->points[atomCount][0])); 
 
	    extStr(temp, line, 39, 46); 
	    sscanf(temp, "%f", &(myobject->points[atomCount][1])); 
 
	    extStr(temp, line, 47, 54); 
	    sscanf(temp, "%f", &(myobject->points[atomCount][2])); 
 
	    extStr(temp, line, 23, 26); 
	    sscanf(temp, "%d", &(myobject->aaNums[atomCount])); 

	    resChains[atomCount] = line[21];

	    atomCount++; 
	  } 	  
	} 
      }	  	   
    } 

    else if (strcmp(record, "TER   ") == 0) { 
      if ((chain[fnum] != ' ') && (chain[fnum] == resChains[atomCount-1]))
	readingDone = 1; 
    } 
  
    else if (strcmp(record, "ENDMDL") == 0) { 
      readingDone = 1; 
    } 
 
    else if (strcmp(record, "END   ") == 0) { 
      readingDone = 1; 
    } 
 
  } 
 
  if (atomCount == 0) { 
 
    printf("\nSorry. No CA atoms with chain identifier \"%c\" found in file %s\n\n", chain[fnum], username[fnum]);
    return NULL;
    
  } 
   
  else { 


    myobject->numPts = atomCount; 

    myobject->ssArray = (int *)malloc((sizeof(int))*(MAXLENGTH)); 
    myobject->sstype = (char *) malloc((sizeof(char))*(myobject->numPts + 10)); 
    arraylen = 0; 
     
    for (i = 0; i < myobject->numPts; i++) { 
      myobject->sstype[i] = 'U'; 
    } 

    myobject->sstype[i] = '\0';
    myobject->objSeq[atomCount] = '\0';  

    for (i = 0; i < atomCount; i++) { 
 
      count = 0; 
 
      for (j = 0; ((j < numhelices) && (count == 0)); j++) { 
 
	if ((myobject->aaNums[i] >= helices[j][0]) && (myobject->aaNums[i] <= helices[j][1]) && (resChains[i] == helixChains[j])) { 
 
	  while ((myobject->aaNums[i+count] <= helices[j][1]) && (i+count < atomCount)) 
	    count++; 
 
	  if (count >= 4) { 
	    for (k = 0; k < count; k++) { 
	      myobject->sstype[i+k] = 'H'; 
	    } 
 
	    myobject->ssArray[arraylen*2] = i; 
	    myobject->ssArray[arraylen*2 + 1] = i + count; 
	    arraylen++; 
	  } 
	  i = i + count - 1; 
	} 
      } 
 
 
      for (j = 0; ((j < numsheets) && (count == 0)); j++) { 
	 
	if ((myobject->aaNums[i] >= sheets[j][0]) && (myobject->aaNums[i] <= sheets[j][1]) && (resChains[i] == sheetChains[j])){ 
 
	  while ((myobject->aaNums[i+count] <= sheets[j][1]) && (i+count < atomCount)) 
	    count++; 
 
	  if (count >= 3) { 
	    for (k = 0; k < count; k++) { 
	      myobject->sstype[i+k] = 'S'; 
	    } 
 
	    myobject->ssArray[arraylen*2] = i; 
	    myobject->ssArray[arraylen*2 + 1] = i + count; 
	    arraylen++; 
	  } 
	  i = i + count - 1; 
	} 
      }	       
    }

    myobject->ssArrayLength = arraylen;

    for (i = 0; i < myobject->numPts; i++) {
      t0 += myobject->points[i][0];
      t1 += myobject->points[i][1];
      t2 += myobject->points[i][2];
      
      myobject->pairNums[i] = -1;
      
    }
    
    tempfloat = 1.0/((GLfloat)(myobject->numPts));
    
    tempfloat *= -1.0;
    
    t0 *= tempfloat;
    t1 *= tempfloat;
    t2 *= tempfloat;
    
    myobject->objMat[0] = 1.0;
    myobject->objMat[1] = 0.0;
    myobject->objMat[2] = 0.0;
    myobject->objMat[3] = 0.0;
    
    myobject->objMat[4] = 0.0;
    myobject->objMat[5] = 1.0;
    myobject->objMat[6] = 0.0;
    myobject->objMat[7] = 0.0;
    
    myobject->objMat[8] = 0.0;
    myobject->objMat[9] = 0.0;
    myobject->objMat[10] = 1.0;
    myobject->objMat[11] = 0.0;
    
    myobject->objMat[12] = t0;
    myobject->objMat[13] = t1;
    myobject->objMat[14] = t2;
    myobject->objMat[15] = 1.0;
        
    myobject->center[0] = 0.0;
    myobject->center[1] = 0.0;
    myobject->center[2] = 0.0;

#ifdef DISPLAYON
    if (globalStartDisplay) {
      myobject->modelNumber = glGenLists(1);
      myobject->color[0] = colorTable[fnum][0];
      myobject->color[1] = colorTable[fnum][1];
      myobject->color[2] = colorTable[fnum][2];
            
      createObjList(myobject);
    }
#endif
    fclose(infile); 
    return myobject;
    

  } 

} 
 
 
OBJ* noSSinPDBFile(int fnum) { 
 
  char syscommand[200]; 
  char line[200];
  OBJ* tempObj;
  char tempname[200];
  
  if (newfile[fnum] == 1) {
    strcpy(tempname, filename[fnum]);

    printf("No secondary structure information in file %s\n", username[fnum]);
    printf("Will now try to create a DSSP file for %s\n\n", username[fnum]);

    filename[fnum][0] = '\0';
    sprintf(filename[fnum], "/tmp/%d.dssp%d", mypid, fnum);
    
    strcpy(syscommand, dsspprogram);
    strcat(syscommand, tempname);
    strcat(syscommand, " ");
    strcat(syscommand, filename[fnum]);

    system(syscommand);

    filePtrs[fnum] = NULL;

    filePtrs[fnum] = fopen(filename[fnum], "r");

    if (filePtrs[fnum] != NULL) {
      
      fgets(line, 200, filePtrs[fnum]); 
      
      if ((line[51] == 'D') && (line[52] == 'S') && (line[53] == 'S') && (line[54] == 'P')) { 
	tempObj = objectFromDSSPFile(fnum); 

	strcpy(syscommand, "rm ");
	strcat(syscommand, filename[fnum]);
	system(syscommand); 

	return tempObj;

      } 
      
      else { 
	printf("Sorry.  Could not create a DSSP file for this protein.\n"); 
	return NULL; 
      } 
    }      
    else { 
      printf("Sorry.  Could not create a DSSP file for this protein.\n"); 
      return NULL; 
    } 
  }
  else {
    printf("No secondary structure information in file %s\n", pdbname[fnum]);
    printf("Will now try to find a DSSP file for %s.\n\n", pdbname[fnum]); 
   
    strcpy(syscommand, "rm ");
    strcat(syscommand, filename[fnum]);
    system(syscommand);
    
    filename[fnum][0] = '\0';
    sprintf(filename[fnum], "/tmp/%d.dssp%d", mypid, fnum);
    
    strcpy(syscommand, lockdir);
    strcat(syscommand, "findDSSPfile.pl ");
    strcat(syscommand, pdbname[fnum]);
    strcat(syscommand, " ");
    strcat(syscommand, filename[fnum]);

    system(syscommand);

    filePtrs[fnum] = NULL;

    filePtrs[fnum] = fopen(filename[fnum], "r");

    if (filePtrs[fnum] != NULL) {
      
      fgets(line, 200, filePtrs[fnum]); 
      
      if ((line[51] == 'D') && (line[52] == 'S') && (line[53] == 'S') && (line[54] == 'P')) { 
	return objectFromDSSPFile(fnum); 
      } 
      
      else { 
	printf("Sorry.  Could not find a DSSP file for this protein.\n"); 
	return NULL; 
      } 
    }      
    else { 
      printf("Sorry.  Could not find a DSSP file for this protein.\n"); 
      return NULL; 
    } 
  } 
    
} 
 
 
 
 
OBJ* objectFromFile(int fnum) { 
 
  char line[200]; 

  rewind(filePtrs[fnum]);

  fgets(line, 200, filePtrs[fnum]); 
   
  if ((line[51] == 'D') && (line[52] == 'S') && (line[53] == 'S') && (line[54] == 'P')) { 
     
    return objectFromDSSPFile(fnum); 
 
  } 
   
  else if ((line[0] == 'H') && (line[1] == 'E') && (line[2] == 'A') && (line[3] == 'D') && (line[4] == 'E') && (line[5] == 'R')) { 
     
    return objectFromPDBFile(fnum); 
 
  } 
  else { 
    fclose(filePtrs[fnum]);
    printf("Sorry.  The input file does not appear to be in either PDB or DSSP format\n"); 
    return NULL;
  }  
} 
 
